﻿using System;
using System.Collections.Generic;
using System.Data.OleDb;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Xml.Serialization;
using UniversalPlantViewerBuilderPlugin.Version1;

namespace GetDataFromMDB
{
    public class Plugin : IPluginVersion1
    {
        private readonly Config _config;

        public Plugin()
        {
            _config = ReadXmlConfig();
        }

        public void BeforeExport(IPreExportHook hook)
        {
            hook.BuilderContext.Log("plugin: GetDataFromMDB started..");

            var mdbFilePath = _config.MdbFilePath;
            if (!File.Exists(mdbFilePath))
            {
                throw new ArgumentException($"Mdb file does not exist: '{mdbFilePath}'", nameof(mdbFilePath));
            }

            var connectionString =
                $"Provider=Microsoft.ACE.OLEDB.12.0;Data Source={_config.MdbFilePath};Persist Security Info = False;";

            try
            {
                using (var connection = new OleDbConnection(connectionString))
                {
                    connection.Open();

                    using (var command = new OleDbCommand(null, connection))
                    {
                        foreach (var element in hook.BuilderElementAccessor.IterateElements())
                        {
                            var elementAttributes = hook.BuilderAttributeAccessor.GetAttributes(element,
                                new HashSet<string> {"Task", "FileName", "UPVC FileName"});
                            var elementTask = (from a in elementAttributes
                                where a.Key == "Task"
                                select a.Value).FirstOrDefault();

                            if (elementTask == "Piping")
                            {
                                command.CommandText = $"select [IdentNumber] from IdentNumbers where OID = '{element.Uid}'";

                                if (_config.Debug)
                                {
                                    hook.BuilderContext.Log("SQL: " + command.CommandText);
                                }

                                var identNumber = Convert.ToString(command.ExecuteScalar());
                                if (!string.IsNullOrWhiteSpace(identNumber))
                                {
                                    hook.BuilderAttributeAccessor.AddAttribute(element, "SDK IdentNumber", identNumber);
                                }
                            }
                            else
                            {
                                if (_config.Debug)
                                {
                                    hook.BuilderContext.Log("plugin: skipping element: " + element.Uid);
                                }
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                hook.BuilderContext.Log("plugin: GetDataFromMDB ERROR!");
                hook.BuilderContext.Log(ex.Message);
            }

            hook.BuilderContext.Log("plugin: GetDataFromMDB finished.");
        }

        private static Config ReadXmlConfig()
        {
            var path = Path.GetDirectoryName(Uri.UnescapeDataString(new UriBuilder(Assembly.GetExecutingAssembly().CodeBase).Path));
            if (path == null)
            {
                throw new InvalidOperationException();
            }

            var reader = new XmlSerializer(typeof(Config));
            using (var file = new StreamReader(Path.Combine(path, "GetDataFromMDB.xml")))
            {
                return (Config) reader.Deserialize(file);
            }
        }

        public int ProcessOrder => 1;
        public string Name => "GetDataFromMDB";
    }
}