﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text.RegularExpressions;
using UniversalPlantViewerBuilderPlugin.Version1;

namespace UPVBPluginLinks
{
    public class Plugin : IPluginVersion1
    {
        public void BeforeExport(IPreExportHook hook)
        {
            hook.BuilderContext.Log($"{Name}: started...");

            hook.BuilderGraphicAccessor.CreateAspect("Spheres");

            // Preparation
            double Diameter = 5.0;
            double x = 0;
            double y = 0;
            double z = 0;

            int aspectNumber = 1;
            string aspectName = "Spheres";
            string DisplayName = "myLink";
            string Url = "http://www.google.de";

            var sessions = hook.BuilderElementAccessor.Sessions;
            var defaultSession = sessions.FirstOrDefault();
            var element = hook.BuilderElementAccessor.CreateElement(Guid.NewGuid(), defaultSession);

            // Create sphere
            var primitives = new List<string>() { $"{aspectNumber} SetDiffuseColor 1 1 1 0.3", $"{aspectNumber} PlaceSphere {Diameter.ToString(CultureInfo.InvariantCulture)} 0 0 0 {x} {y} {z}" };
            hook.BuilderGraphicAccessor.CreateGraphic(element, aspectName, primitives);
            // Create Link
            var link = hook.BuilderLinkAccessor.CreateLink(Url, new Color() { R = 126, G = 140, B = 226, A = 255 }, DisplayName);

            // Add link to sphere
            hook.BuilderLinkAccessor.AddRelation(link, element);

            // Add attributes to sphere
            hook.BuilderContext.Log($"{Name}: Adding attributes");
            hook.BuilderAttributeAccessor.AddAttribute(element, "Sphere Name", DisplayName);
            hook.BuilderAttributeAccessor.AddAttribute(element, "UID", Regex.Replace(DisplayName, "[^A-Z0-9]", ""));
            hook.BuilderAttributeAccessor.AddAttribute(element, "Url", Url);


            hook.BuilderContext.Log($"{Name}: finished...");
        }

        public int ProcessOrder => 1;
        public string Name => "UPVBPluginSpheres";
    }
}