﻿using System;
using CAXperts.Libraries.Database.Core.Excel;
using UniversalPlantViewerBuilderPlugin.Version1;

namespace UPVPluginFixSystemPath
{
    public class Plugin : IPluginVersion1
    {
        public void BeforeExport(IPreExportHook hook)
        {
            hook.BuilderContext.Log($"{Name}: started...");

            try
            {
                var config = new Config();

                using (var connection = new ExcelConnection(config.ConnectionString, true, true))
                {

                    var sql = $"SELECT * FROM [{config.SheetName}$]";
                    using (var reader = connection.CreateReader(sql))
                    {
                        while (reader.Read())
                        {
                            try
                            {
                                var orgName = reader["Original Name"].ToString();
                                var newName = reader["New Name"].ToString();

                                if (string.IsNullOrWhiteSpace(orgName) || string.IsNullOrWhiteSpace(newName))
                                {
                                    continue;
                                }

                                hook.BuilderAttributeAccessor.RenameAttribute(orgName, newName);

                                hook.BuilderContext.Log($"{Name}: Attribute renamed from '{orgName}' to '{newName}'.");
                            }
                            catch (Exception e)
                            {
                                hook.BuilderContext.LogError(e.Message, e);
                            }
                        }
                    }
                }
            }
            catch (Exception e)
            {
                hook.BuilderContext.Log(e.Message);
            }

            hook.BuilderContext.Log($"{Name}: finished...");
        }

        public int ProcessOrder => 1;
        public string Name => "UPVPluginRenameAttributes";

    }
}